//
//  TVOCall.h
//  TwilioVoice
//
//  Copyright © 2018 Twilio, Inc. All rights reserved.
//

@protocol TVOCallDelegate;
@class TVOStatsReport;

/**
 * Enumeration indicating the state of the Call.
 */
typedef NS_ENUM(NSUInteger, TVOCallState) {
    TVOCallStateConnecting = 0, ///< The Call is connecting.
    TVOCallStateRinging,        ///< The Call is ringing.
    TVOCallStateConnected,      ///< The Call is connected.
    TVOCallStateDisconnected    ///< The Call is disconnected.
};

/**
 * `TVOCallGetStatsBlock` is invoked asynchronously when the results of the `[TVOCall getStatsWithBlock:]` method are available.
 *
 * @param statsReports A collection of `TVOStatsReport` objects
 */
typedef void (^TVOCallGetStatsBlock)(NSArray<TVOStatsReport *> * _Nonnull statsReports);

/**
 * The `TVOCall` class represents a bi-directional voice Call. `TVOCall` objects are not created directly; they
 * are returned by `<[TVOCallInvite acceptWithOptions:delegate:]>` or `<[TwilioVoice connectWithOptions:delegate:]>`.
 */
@interface TVOCall : NSObject

/**
 * @name Properties
 */

/**
 * @brief `From` value of the Call.
 *
 * @discussion This may be `nil` if the call object was created by calling the 
 * `<[TwilioVoice connectWithOptions:delegate:]>` method.
 */
@property (nonatomic, strong, readonly, nullable) NSString *from;

/**
 * @brief `To` value of the Call.
 *
 * @discussion This may be `nil` if the call object was created by calling the 
 * `<[TwilioVoice connectWithOptions:delegate:]>` method.
 */
@property (nonatomic, strong, readonly, nullable) NSString *to;

/**
 * @brief A server assigned identifier (SID) for the Call.
 *
 * @discussion A SID is a globally unique identifier which can be very useful for debugging Call traffic. The Call sid
 * may be `nil` until the call is in `TVOCallStateRinging` state.
 */
@property (nonatomic, strong, readonly, nonnull) NSString *sid;

/**
 * @brief Property that defines if the Call is muted.
 *
 * @discussion The Voice SDK's media engine will start sending silent audio frames when the Call is muted. Setting the
 * property will only take effect if the `<state>` is `TVOCallStateConnected`.
 */
@property (nonatomic, assign, getter=isMuted) BOOL muted;

/**
 * @brief The current state of the Call.
 *
 * @discussion All `TVOCall` instances start in `TVOCallStateConnecting` and end in `TVOCallStateDisconnected`.
 * After creation, a Call will transition to `TVOCallStateConnected` if successful or `TVOCallStateDisconnected` if the
 * connection attempt fails.
 *
 * @see TVOCallState
 */
@property (nonatomic, assign, readonly) TVOCallState state;

/**
 * @brief Property that defines if the Call is on hold.
 *
 * @discussion If the Call is on hold, The Voice SDK's media engine will send silent audio frames to the remote party,
 * and remote party's audio playback will be disabled.
 */
@property (nonatomic, getter=isOnHold) BOOL onHold;

/**
 * @name General Call Actions
 */

/**
 * @brief Disconnects the Call.
 *
 * @discussion Calling this method on a `TVOCall` that is in the `<state>` `TVOCallStateDisconnected` has no effect.
 */
- (void)disconnect;

/**
 * @brief Send a string of digits.
 *
 * @discussion Calling this method on a `TVOCall` that is not in the `<state>` `TVOCallStateConnected` has no effect.
 *
 * @param digits A string of characters to be played. Valid values are '0' - '9', '*', '#', and 'w'.
 *               Each 'w' will cause a 500 ms pause between digits sent.
 */
- (void)sendDigits:(nonnull NSString *)digits;

/**
 * @brief Retrieve stats for the audio track.
 *
 * @param block The block to be invoked when the stats are available.
 */
- (void)getStatsWithBlock:(nonnull TVOCallGetStatsBlock)block;

- (null_unspecified instancetype)init __attribute__((unavailable("A `TVOCall` object cannot be instantiated directly. See `[TVOCallInvite acceptWithOptions:delegate:]` or `[TwilioVoice connectWithOptions:delegate:]`")));

@end


/**
 *  CallKit specific additions.
 */
@interface TVOCall (CallKit)

/**
 * @brief A unique identifier for the Call.
 *
 * @discussion Use this UUID as an argument to CallKit methods.
 * You can provide a UUID for outgoing calls using `<[TwilioVoice connectWithOptions:delegate:]>`.
 * Calls created via `<[TVOCallInvite acceptWithOptions:delegate:]>` inherit their `uuid` from the Invite itself.
 */
@property (nonatomic, strong, readonly, nonnull) NSUUID *uuid;

@end
